/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.bonita;

import hero.interfaces.BnNodePropertyValue;
import hero.interfaces.BnProjectPropertyValue;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.util.HeroException;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.xml.transform.TransformerException;

import org.objectweb.petals.common.util.XMLUtil;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Bean with project and activity properties.
 * 
 * @version $Rev: $ $Date: $
 * @author Guillaume Decarnin - Open Wide
 *
 */
public class BonitaProject {

    private String login = "";
    private String projectName = "";
    private String projectVersion = "";
    private String instanceName = "";
    private String activityName = "";
    private String activityState = "";
    private Map<String, String> projectProperties = new HashMap<String, String>();
    private Map<String, String> activityProperties = new HashMap<String, String>();
    
    public void reset() {
        projectName = "";
        projectVersion = "";
        instanceName = "";
        activityName = "";
        activityState = "";
        projectProperties.clear();
        projectProperties.clear();
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getActivityName() {
        return activityName;
    }

    public void setActivityName(String activityName) {
        this.activityName = activityName;
    }

    public Map<String, String> getActivityProperties() {
        return activityProperties;
    }

    public void setActivityProperties(Map<String, String> activityProperties) {
        this.activityProperties = activityProperties;
    }

    public String getInstanceName() {
        return instanceName;
    }

    public void setInstanceName(String instanceName) {
        this.instanceName = instanceName;
    }

    public String getProjectName() {
        return projectName;
    }

    public void setProjectName(String projectName) {
        this.projectName = projectName;
    }

    public String getProjectVersion() {
        return projectVersion;
    }

    public void setProjectVersion(String projectVersion) {
        this.projectVersion = projectVersion;
    }

    public Map<String, String> getProjectProperties() {
        return projectProperties;
    }

    public void setProjectProperties(Map<String, String> projectProperties) {
        this.projectProperties = projectProperties;
    }
    
    public String getActivityState() {
        return activityState;
    }

    public void setActivityState(String state) {
        this.activityState = state;
    }
    
    public void loadData(Node xml) {
        reset();
        
        NodeList children = xml.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            Node child = children.item(i);
            if (child.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            
            if (child.getNodeName().equals("login")) {
                login = XMLUtil.getTextContent(child).trim();
            }
            
            else  if (child.getNodeName().equals("process")) {
                Node projectNameNode = child.getAttributes().getNamedItem("projectName");
                if (projectNameNode != null) {
                    projectName = XMLUtil.getTextContent(projectNameNode);
                }
                
                Node projectVersionNode = child.getAttributes().getNamedItem("projectVersion");
                if (projectVersionNode != null) {
                    projectVersion = XMLUtil.getTextContent(projectVersionNode);
                }
                
                Node instanceNameNode = child.getAttributes().getNamedItem("instanceName");
                if (instanceNameNode != null) {
                    instanceName = XMLUtil.getTextContent(instanceNameNode);
                }
                
                NodeList processChildren = child.getChildNodes();
                for (int y = 0; y < processChildren.getLength(); y++) {
                    Node processChild = processChildren.item(y);
                    if (processChild.getNodeType() != Node.ELEMENT_NODE) {
                        continue;
                    }
                    try {
                        String value = BonitaXmlUtil.createStringFromNodeList(processChild.getChildNodes());
                        projectProperties.put(processChild.getNodeName(), value);
                    } catch (TransformerException e) {
                        e.printStackTrace();
                    }
                }
            }
            
            else if (child.getNodeName().equals("activity")) {
                Node activityNameNode = child.getAttributes().getNamedItem("activityName");
                if (activityNameNode != null) {
                    activityName = XMLUtil.getTextContent(activityNameNode);
                }
                
                NodeList activityChildren = child.getChildNodes();
                for (int y = 0; y < activityChildren.getLength(); y++) {
                    Node activityChild = activityChildren.item(y);
                    if (activityChild.getNodeType() != Node.ELEMENT_NODE) {
                        continue;
                    }
                    try {
                        String value = BonitaXmlUtil.createStringFromNodeList(activityChild.getChildNodes());
                        activityProperties.put(activityChild.getNodeName(), value);
                    } catch (TransformerException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    public void loadProjectProperties(ProjectSession projectSession) throws RemoteException, HeroException {
        getProjectProperties().clear();
        Collection<BnProjectPropertyValue> projectProperties = projectSession.getProperties();
        for (BnProjectPropertyValue entry : projectProperties) {
            getProjectProperties().put(entry.getTheKey(), entry.getTheValue());
        }
    }

    @SuppressWarnings("unchecked")
    public void loadActivityProperties(ProjectSession projectSession) throws RemoteException, HeroException {
        getActivityProperties().clear();
        Collection<BnNodePropertyValue> activityProperties = projectSession.getNodeProperties(getActivityName());
        for (BnNodePropertyValue entry : activityProperties) {
            getActivityProperties().put(entry.getTheKey(), entry.getTheValue());
        }
    }

    public void loadProjectName(ProjectSession projectSession) throws RemoteException, HeroException {
        setProjectName(projectSession.getProjectNameOfInstance(getInstanceName()));
    }

    public void loadActivityState(ProjectSession projectSession) throws RemoteException, HeroException {
        projectSession.initProject(getInstanceName());
        int state = projectSession.getNodeState(getActivityName());
        String stateName =  Constants.Nd.nodeStateName[state];
        setActivityState(stateName);
    }

    /*
    <content>
        <process projectName="..." instanceName="...">
            <key1>value</key1>
            <key2>value</key2>
        </process>
        <activity activityName="..." projectName="..." instanceName="...">
            <key3>value</key3>
            <key4>value</key4>
        </activity>
    </content>
     */
    public String toXml() {
        StringBuilder content = new StringBuilder();
        
        content.append("<content>\n");
        content.append(processToXml(true));
        content.append(activityToXml(true, true, true));
        content.append("</content>");
        
        return content.toString();
    }
    
    /*
    <activity activityName="..." projectName="..." instanceName="...">
        <key3>value</key3>
        <key4>value</key4>
    </activity>
     */
    public String activityToXml(boolean withProjectName, boolean withInstanceName, boolean withState) {
        StringBuilder xml = new StringBuilder();
        
        xml.append("\t<activity activityName=\"");
        xml.append(getActivityName());
        xml.append("\"");
        
        if (withProjectName) {
            xml.append(" projectName=\"");
            xml.append(getProjectName());
            xml.append("\"");
        }
        
        if (withInstanceName) {
            xml.append(" instanceName=\"");
            xml.append(getInstanceName());
            xml.append("\"");
        }
        
        if (withState) {
            xml.append(" state=\"");
            xml.append(getActivityState());
            xml.append("\"");
        }
        
        if (getActivityProperties().size() > 0) {
            xml.append(">\n");
            addXmlProperties(xml, getActivityProperties());
            xml.append("\t</activity>\n");
        } else {
        	xml.append("/>\n");
        }
        
        return xml.toString();
    }

    /*
    <process projectName="..." instanceName="...">
        <key1>value</key1>
        <key2>value</key2>
    </process>
     */
    public String processToXml(boolean withInstanceName) {
        StringBuilder xml = new StringBuilder();

        xml.append("\t<process projectName=\"");
        xml.append(getProjectName());
        xml.append("\"");
        
        if (withInstanceName) {
            xml.append(" instanceName=\"");
            xml.append(getInstanceName());
            xml.append("\"");
        }
        
        if (getProjectProperties().size() > 0) {
            xml.append(">\n");
            addXmlProperties(xml, getProjectProperties());
            xml.append("\t</process>\n");
        } else {
        	xml.append("/>\n");
        }
        
        return xml.toString();
    }

    private static void addTag(StringBuilder string, String tagName, String text) {
        string.append("\t\t<");
        string.append(tagName);
        string.append(">");
        
        string.append(text);
        
        string.append("</");
        string.append(tagName);
        string.append(">\n");
    }

    private static void addXmlProperties(StringBuilder string, Map<String, String> properties) {
        for (Entry<String, String> entry : properties.entrySet()) {
            addTag(string, entry.getKey(), entry.getValue());
        }
    }
}
