/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.bonita;

import static org.ow2.petals.se.bonita.BonitaEngine.LOGIN_CONTEXT_NAME_KEY;
import static org.ow2.petals.se.bonita.BonitaEngine.LOGIN_KEY;
import static org.ow2.petals.se.bonita.BonitaEngine.PASSWORD_KEY;
import static org.ow2.petals.se.bonita.BonitaEngine.RMI_URL_KEY;
import static org.ow2.petals.se.bonita.BonitaEngine.ROLE_KEY;

import java.util.ArrayList;
import java.util.Map;

import org.objectweb.petals.common.descriptor.Extensions;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.su.ServiceUnitExtensionsValidator;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;

/**
 * SU extensions validator for Bonita.
 * @version $Rev: $ $Date: $
 * @author Guillaume Decarnin - Open Wide
 *
 */
public class BonitaServiceUnitExtensionsValidator implements ServiceUnitExtensionsValidator {
    
    public void validateConsumesExtensions(Extensions extensions) throws PEtALSComponentSDKException {
        // nothing to do
    }

    public void validateProvidesExtensions(Extensions extensions) throws PEtALSComponentSDKException {
        // validate mandatory keys
        ArrayList<String> mandatoryKeys = new ArrayList<String>();
        mandatoryKeys.add(LOGIN_CONTEXT_NAME_KEY);
        mandatoryKeys.add(RMI_URL_KEY);
        
        ArrayList<String> loginIds = new ArrayList<String>();
        
        // validate logins and passwords
        Map<String, String> keysValues = PetalsExtensionsUtil.getExtensionsKeysValues(extensions);
        for (Map.Entry<String, String> entry : keysValues.entrySet()) {
        	if (entry.getKey().startsWith(LOGIN_KEY) && !entry.getKey().equals(LOGIN_CONTEXT_NAME_KEY)) {
        		String id = entry.getKey().substring(LOGIN_KEY.length());
        		loginIds.add(id);
        	}
        }
        
        // do validation
        for (String key : mandatoryKeys) {
            checkProperty(extensions, key);
        }
        for (String id : loginIds) {
        	checkLoginIdProperty(extensions, id);
        }
    }

    private void checkProperty(Extensions extensions, String key) throws PEtALSComponentSDKException {
        if (!PetalsExtensionsUtil.hasKeyValueExtension(extensions, key)) {
            throw new PEtALSComponentSDKException("Bonita SU configuration error: the key-value property \"" + key + "\" is missing.");
        }
    }

    private void checkLoginIdProperty(Extensions extensions, String id) throws PEtALSComponentSDKException {
    	String passwordKey = PASSWORD_KEY + id;
    	String roleKey = ROLE_KEY + id;
    	boolean hasPassword = PetalsExtensionsUtil.hasKeyValueExtension(extensions, passwordKey);
    	boolean hasRole = PetalsExtensionsUtil.hasKeyValueExtension(extensions, roleKey);
        if (!hasPassword && !hasRole) {
            throw new PEtALSComponentSDKException("Bonita SU configuration error: the key-value property \"" + passwordKey + "\" or \"" + roleKey + "\" is missing.");
        }
    }
}
