/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.bonita;

import java.io.StringWriter;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * XmlUtil for Bonita SE.
 * 
 * @version $Rev: $ $Date: $
 * @author Guillaume Decarnin - Open Wide
 *
 */
public class BonitaXmlUtil {
    
    /**
     * Create a DOM Node result from an XML Source
     * 
     * @param source the XML Source
     * @return a DOM representation of the XML Document
     * @throws TransformerException
     */
    public static Node createDOMNodeFromSource(Source source) throws TransformerException {
        DOMResult resultDom = new DOMResult();
        TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        transformer.transform(source, resultDom);
        Node resultNode = resultDom.getNode();
        return resultNode;
    }

    /**
     * Create a String result from DOM NodeList
     *
     * @param nodes
     *            the DOM Nodes. Must not be null
     * @return a String representation of the DOM Document
     * @throws TransformerException
     */
    public static String createStringFromNodeList(NodeList nodes) throws TransformerException {
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        final TransformerFactory tFactory = TransformerFactory.newInstance();
        Transformer transformer;
        transformer = tFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        
        for (int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            node.normalize();
            final Source source = new DOMSource(node);
            transformer.transform(source, resultStream);
        }
        
        return out.toString();
    }

}
