/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id : $ 
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.bonita.security;

import java.io.Serializable;
import java.security.Principal;

public class PetalsSeBonitaDefaultPrincipal implements Principal, Serializable {
	private static final long serialVersionUID = 7637651358046402553L;
	
	/**
	 * @serial
	 */
	private String name;

	/**
	 * Create a SamplePrincipal with a Sample username.
	 *
	 * @param name the Sample username for this user.
	 *
	 * @exception NullPointerException if the <code>name</code>
	 *			is <code>null</code>.
	 */
	public PetalsSeBonitaDefaultPrincipal(String name) {
		if (name == null) {
			throw new NullPointerException("illegal null input");
		}
		this.name = name;
	}

	/**
	 * Return the Sample username for this <code>SamplePrincipal</code>.
	 *
	 * @return the Sample username for this <code>SamplePrincipal</code>
	 */
	public String getName() {
		return name;
	}

	/**
	 * Return a string representation of this <code>SamplePrincipal</code>.
	 *
	 * @return a string representation of this <code>SamplePrincipal</code>.
	 */
	@Override
	public String toString() {
		return("PetalsSeBonitaDefaultPrincipal:" + name);
	}

	/**
	 * Compares the specified Object with this <code>SamplePrincipal</code>
	 * for equality.  Returns true if the given object is also a
	 * <code>SamplePrincipal</code> and the two SamplePrincipals
	 * have the same username.
	 *
	 * @param o Object to be compared for equality with this
	 *		<code>SamplePrincipal</code>.
	 *
	 * @return true if the specified Object is equal equal to this
	 *		<code>SamplePrincipal</code>.
	 */
	@Override
	public boolean equals(Object o) {
		if (o == null)
			return false;

		if (this == o)
			return true;

		if (!(o instanceof PetalsSeBonitaDefaultPrincipal))
			return false;
		
		PetalsSeBonitaDefaultPrincipal that = (PetalsSeBonitaDefaultPrincipal)o;
		
		return this.getName().equals(that.getName());
	}

	/**
	 * Return a hash code for this <code>SamplePrincipal</code>.
	 *
	 * @return a hash code for this <code>SamplePrincipal</code>.
	 */
	@Override
	public int hashCode() {
		return name.hashCode();
	}
}
